"
 COPYRIGHT (c) 1997 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice. This software may not
 be provided or otherwise made available to, or used by, any
 other person. No title to or ownership of the software is
 hereby transferred.
"



"{ Package: 'stx:libview2' }"

ApplicationModel subclass:#ToolApplicationModel
	instanceVariableNames:'timeBlock'
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Framework'
!

ToolApplicationModel class instanceVariableNames:'history clipboard settings showingHelp instances'

"
 The following class instance variables are inherited by this class:

	ApplicationModel - ClassResources
	Model - 
	Object - 
"
!

!ToolApplicationModel class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1997 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice. This software may not
 be provided or otherwise made available to, or used by, any
 other person. No title to or ownership of the software is
 hereby transferred.
"


!

documentation
"
    Framework class for tool applications (UIPainter, MenuEditor etc.).
    Provides hooks for modifiedChannels, history management,
    showing help texts, about menu, help menu etc.

    [author:]
        Thomas Zwick, eXept Software AG
"
! !

!ToolApplicationModel class methodsFor:'accessing'!

allToolInstances
    ^ instances ? #()

    "Created: / 4.2.1999 / 15:34:55 / cg"
!

authorLinesForAboutBox
    "extracts the author(s) from the documentation method."

    |firstClassWithDocumentation src s line lines|

    firstClassWithDocumentation := self class withAllSuperclasses detect:[:metacls| metacls includesSelector:#documentation].
    firstClassWithDocumentation notNil ifTrue:[
        (firstClassWithDocumentation inheritsFrom:ToolApplicationModel class) ifTrue:[
            src := (firstClassWithDocumentation compiledMethodAt:#documentation) source.
            src notNil ifTrue:[
                s := src readStream.
                s upToAll:'[author:]'.
                s nextLine.   "/ skip to end
                line := s nextLine.   "/ line after [author:]
                lines := OrderedCollection new.
                [line notNil and:[line notEmpty]] whileTrue:[
                    line := line trimBlanks.
                    line size > 1 ifTrue:[
                        lines add:line trimBlanks.
                        line := s nextLine.
                    ] ifFalse:[
                        line := nil
                    ]
                ].
                ^ lines.
            ]
        ]
    ].
    ^ #('unnown')

    "Created: / 1.2.1998 / 15:41:43 / cg"
    "Modified: / 1.2.1998 / 15:59:10 / cg"
!

forgetInstance:anInstance
    instances remove:anInstance.
    instances isEmpty ifTrue:[instances := nil].

    "Modified: / 4.2.1999 / 15:38:49 / cg"
!

label
    "returns the label for the tool instances of this class;
     extracted from the class name;
     can be redefined in subclasses"

    |label|
    label := ''.
    self name do:
    [:c|
        c isUppercase ifTrue: [label := label, $ ].
        label := label, c
    ].
    ^label trimBlanks

    "Modified: / 31.7.1998 / 18:04:18 / cg"
!

rememberInstance:anInstance
    instances isNil ifTrue:[
        instances := WeakIdentitySet new
    ].
    instances add:anInstance

    "Created: / 4.2.1999 / 15:33:43 / cg"
    "Modified: / 4.2.1999 / 15:34:14 / cg"
!

settings
    "returns the settings dictionary"

    settings isNil ifTrue: [settings := IdentityDictionary new].
    ^settings
!

showingHelp
    "returns true if active help is turned on for instances of this class"

    showingHelp isNil ifTrue: [showingHelp := true].
    ^showingHelp

    "Created: / 31.7.1998 / 18:03:38 / cg"
    "Modified: / 31.7.1998 / 18:16:55 / cg"
!

showingHelp: aBoolean
    "enable/disable active help for instances of this class"

    ^ showingHelp := aBoolean

    "Modified: / 31.7.1998 / 18:17:14 / cg"
! !

!ToolApplicationModel class methodsFor:'clipboard'!

clipboard
    "returns the clipboard of this tool class"

    ^clipboard
!

clipboard: anEditObject
    "sets the clipboard for this tool class"

    clipboard := anEditObject
! !

!ToolApplicationModel class methodsFor:'help'!

openAboutSTX
    "opens an about box (on ST/X)"

    |box|

    AboutBox isNil ifTrue:[
        "/ this handles bad installations of ST/X,
        "/ where the AboutBox is missing.
        "/ (May vanish in the future)
        ^ self information:('Smalltalk/X rev.' , Smalltalk versionString)
    ].

    box := AboutBox new.
    box autoHideAfter:10 with:[].
    box showAtCenter
"/    box showAtPointer

    "Modified: / 14.8.1998 / 13:03:58 / cg"
    "Created: / 14.8.1998 / 13:17:43 / cg"
! !

!ToolApplicationModel class methodsFor:'help specs'!

flyByHelpSpec
    <resource: #help>

    ^super flyByHelpSpec addPairsFrom:#(

"/#commitCancel
"/'Cancels'
"/
"/#commitOK
"/'Commit'

#editCopy
'Copy to clipboard'

#editCut
'Move to clipboard'

#editDelete
'Delete'

#editMoveDown
'Move down'

#editMoveIn
'Move into next'

#editMoveInAbove
'Move into previous'

#editMoveOut
'Move out'

#editMoveUp
'Move up'

#editPaste
'Paste'

#editUndo
'Undo'

#fileBrowseClass
'Browse class'

#fileDefineClass
'Define class'

#fileDefineClassAndSelector
'Define class & selector'

#fileExit
'Exit'

#helpShowHelp
'Toggle help'
)
!

helpSpec
    "This resource specification was automatically generated
     by the UIHelpTool of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the UIHelpTool may not be able to read the specification."

    "
     UIHelpTool openOnClass:ToolApplicationModel    
    "

    <resource: #help>

    ^super helpSpec addPairsFrom:#(

#about
'Info functions.'

#aboutSTX
'Info about the running ST/X.'

#aboutThisAppliaction
'Info about that applications release and author.'

#add
'Add functions.'

#commitCancel
'Cancels changes.'

#commitOK
'Commit changes.'

#edit
'Edit functions.'

#editCopy
'Copy the selected item(s) to the clipboard.'

#editCut
'Move the selected item(s) to the clipboard.'

#editDelete
'Delete the selected item(s).'

#editMoveDown
'Move the selected item one step down (in the order list).'

#editMoveIn
'Move the selected item into the next item (as child item).'

#editMoveInAbove
'Move the selected item into the previous item (as child item).'

#editMoveOut
'Move the selected item out of its parent item.'

#editMoveUp
'Move the selected item one step up (in the order list).'

#editPaste
'Paste from the clipboard.'

#editUndo
'Undo the last action.'

#file
'File functions.'

#fileBrowseClass
'Open a System Browser on the current class.'

#fileDefineClass
'Open a dialog to define the current class.'

#fileDefineClassAndSelector
'Open a dialog to define the current class and selector.'

#fileExit
'Exit this application.'

#generate
'Code generation.'

#help
'Help functions.'

#helpHelpTool
'Show the Help Tools documentation.'

#helpShowHelp
'Toggle display of help texts.'

#helpTutorial
'Show this applications documentation.'

#history
'History functions.'

#historyEmptyMenu
'Clear the history.'

#historyMenuItem
'Switch to that item.'

#settings
'Settings functions.'

#test
'Test functions.'

)
! !

!ToolApplicationModel class methodsFor:'history'!

getHistory
    "returns the history for this tool class"

    history isNil ifTrue: [history := OrderedCollection new].
    ^history

!

historyMaxSize
    "returns the maximum size for the history of this tool class"

    ^10

! !

!ToolApplicationModel class methodsFor:'image specs'!

desktopIcon
    "returns the icon used for the desktop"

    <resource: #programImage>

    ^Icon
        constantNamed:#'ToolApplicationModel desktopIcon'
        ifAbsentPut:
        [
            |desktopIcon|
            desktopIcon := Icon stxIcon.
            desktopIcon magnifiedBy: Display preferredIconSize/desktopIcon extent
        ]

    "Modified: / 24.8.1998 / 21:34:38 / cg"
!

menuIcon
    "returns the icon used for the menu bar"

    <resource: #programImage>

    ^Icon
        constantNamed:#'ToolApplicationModel menuIcon'
        ifAbsentPut:
        [
            |menuIcon myDevice|
            menuIcon := Icon stxIcon.
            myDevice := Screen current.
            menuIcon magnifiedBy: (MenuPanel defaultFont heightOn:myDevice)/menuIcon height
        ]

    "Modified: / 24.8.1998 / 21:34:33 / cg"
! !

!ToolApplicationModel class methodsFor:'interface specs'!

windowSpecForCommit
    "This resource specification was automatically generated
     by the UIPainter of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the UIPainter may not be able to read the specification."

    "
     UIPainter new openOnClass:ToolApplicationModel andSelector:#windowSpecForCommit
     ToolApplicationModel new openInterface:#windowSpecForCommit
    "

    <resource: #canvas>

    ^ 
     #(#FullSpec
        #name: #windowSpecForCommit
        #window: 
       #(#WindowSpec
          #label: 'Commit Buttons'
          #name: 'Commit Buttons'
          #min: #(#Point 100 22)
          #max: #(#Point 1152 900)
          #bounds: #(#Rectangle 18 51 318 111)
        )
        #component: 
       #(#SpecCollection
          #collection: #(
           #(#HorizontalPanelViewSpec
              #name: 'HorizontalPanel1'
              #layout: #(#LayoutFrame 0 0.0 0 0.0 0 1.0 0 1.0)
              #horizontalLayout: #fitSpace
              #verticalLayout: #centerMax
              #horizontalSpace: 3
              #verticalSpace: 3
              #reverseOrderIfOKAtLeft: true
              #component: 
             #(#SpecCollection
                #collection: #(
                 #(#ActionButtonSpec
                    #label: 'Cancel'
                    #name: 'cancelButton'
                    #activeHelpKey: #commitCancel
                    #translateLabel: true
                    #resizeForLabel: true
                    #tabable: true
                    #model: #cancel
                    #enableChannel: #valueOfEnablingCommitButtons
                    #useDefaultExtent: true
                  )
                 #(#ActionButtonSpec
                    #label: 'OK'
                    #name: 'okButton'
                    #activeHelpKey: #commitOK
                    #translateLabel: true
                    #resizeForLabel: true
                    #tabable: true
                    #model: #accept
                    #enableChannel: #valueOfEnablingCommitButtons
                    #useDefaultExtent: true
                  )
                 )
               
              )
            )
           )
         
        )
      )
!

windowSpecForCommitWithoutChannels
    "This resource specification was automatically generated
     by the UIPainter of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the UIPainter may not be able to read the specification."

    "
     UIPainter new openOnClass:ToolApplicationModel andSelector:#windowSpecForCommitWithoutChannels
     ToolApplicationModel new openInterface:#windowSpecForCommitWithoutChannels
    "

    <resource: #canvas>

    ^ 
     #(#FullSpec
        #name: #windowSpecForCommitWithoutChannels
        #window: 
       #(#WindowSpec
          #label: 'Commit Buttons Without Channels'
          #name: 'Commit Buttons Without Channels'
          #min: #(#Point 100 22)
          #max: #(#Point 1152 900)
          #bounds: #(#Rectangle 18 51 318 97)
        )
        #component: 
       #(#SpecCollection
          #collection: #(
           #(#HorizontalPanelViewSpec
              #name: 'HorizontalPanel1'
              #layout: #(#LayoutFrame 0 0.0 0 0.0 0 1.0 0 1.0)
              #horizontalLayout: #fitSpace
              #verticalLayout: #centerMax
              #horizontalSpace: 3
              #verticalSpace: 3
              #reverseOrderIfOKAtLeft: true
              #component: 
             #(#SpecCollection
                #collection: #(
                 #(#ActionButtonSpec
                    #label: 'Cancel'
                    #name: 'cancelButton'
                    #activeHelpKey: #commitCancel
                    #translateLabel: true
                    #resizeForLabel: true
                    #tabable: true
                    #model: #cancel
                    #useDefaultExtent: true
                  )
                 #(#ActionButtonSpec
                    #label: 'OK'
                    #name: 'okButton'
                    #activeHelpKey: #commitOK
                    #translateLabel: true
                    #resizeForLabel: true
                    #tabable: true
                    #model: #accept
                    #useDefaultExtent: true
                  )
                 )
               
              )
            )
           )
         
        )
      )
!

windowSpecForInfoBar
    "This resource specification was automatically generated
     by the UIPainter of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the UIPainter may not be able to read the specification."

    "
     UIPainter new openOnClass:ToolApplicationModel andSelector:#windowSpecForInfoBar
     ToolApplicationModel new openInterface:#windowSpecForInfoBar
    "

    <resource: #canvas>

    ^ 
     #(#FullSpec
        #name: #windowSpecForInfoBar
        #window: 
       #(#WindowSpec
          #label: 'Info Bar'
          #name: 'Info Bar'
          #min: #(#Point 400 320)
          #max: #(#Point 1152 900)
          #bounds: #(#Rectangle 13 23 513 369)
          #menu: #menu
        )
        #component: 
       #(#SpecCollection
          #collection: #(
           #(#ViewSpec
              #name: 'view'
              #layout: #(#LayoutFrame 0 0.0 0 0.0 0 1.0 0 1.0)
              #component: 
             #(#SpecCollection
                #collection: #(
                 #(#LabelSpec
                    #name: 'infoLabel'
                    #layout: #(#LayoutFrame 0 0.0 0 0.0 0 1.0 0 1.0)
                    #level: -1
                    #labelChannel: #valueOfInfoLabel
                    #resizeForLabel: false
                    #adjust: #left
                  )
                 )
               
              )
            )
           )
         
        )
      )
!

windowSpecForInfoBarWithClock
    "This resource specification was automatically generated
     by the UIPainter of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the UIPainter may not be able to read the specification."

    "
     UIPainter new openOnClass:ToolApplicationModel andSelector:#windowSpecForInfoBarWithClock
     ToolApplicationModel new openInterface:#windowSpecForInfoBarWithClock
    "

    <resource: #canvas>

    ^ 
     #(#FullSpec
        #name: #windowSpecForInfoBarWithClock
        #window: 
       #(#WindowSpec
          #label: 'Info Bar'
          #name: 'Info Bar'
          #min: #(#Point 400 320)
          #max: #(#Point 1152 900)
          #bounds: #(#Rectangle 13 23 513 343)
          #menu: #menu
        )
        #component: 
       #(#SpecCollection
          #collection: #(
           #(#VariableHorizontalPanelSpec
              #name: 'VariableHorizontalPanel1'
              #layout: #(#LayoutFrame 0 0 0 0 0 1 0 1)
              #barWidth: 2
              #showHandle: false
              #component: 
             #(#SpecCollection
                #collection: #(
                 #(#LabelSpec
                    #name: 'infoLabel'
                    #level: -1
                    #labelChannel: #valueOfInfoLabel
                    #resizeForLabel: false
                    #adjust: #left
                  )
                 #(#LabelSpec
                    #name: 'timeLabel'
                    #level: -1
                    #labelChannel: #valueOfTimeLabel
                    #resizeForLabel: false
                    #adjust: #rightLeft
                  )
                 )
               
              )
              #handles: #(#Any 0.854 1.0)
            )
           )
         
        )
      )
! !

!ToolApplicationModel class methodsFor:'menu specs'!

menuAbout
    "This resource specification was automatically generated
     by the MenuEditor of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the MenuEditor may not be able to read the specification."

    "
     MenuEditor new openOnClass:ToolApplicationModel andSelector:#menuAbout
     (Menu new fromLiteralArrayEncoding:(ToolApplicationModel menuAbout)) startUp
    "

    <resource: #menu>

    ^
     
       #(#Menu
          
           #(
             #(#MenuItem
                #label: 'About Smalltalk/X...'
                #translateLabel: true
                #value: #openAbout
                #activeHelpKey: #aboutSTX
            )
             #(#MenuItem
                #label: '-'
            )
             #(#MenuItem
                #label: 'About this Application...'
                #translateLabel: true
                #value: #openAboutThisApplication
                #activeHelpKey: #aboutThisAppliaction
            )
          ) nil
          nil
      )

    "Modified: / 19.5.1998 / 21:44:09 / cg"
!

menuHelp
    "This resource specification was automatically generated
     by the MenuEditor of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the MenuEditor may not be able to read the specification."

    "
     MenuEditor new openOnClass:ToolApplicationModel andSelector:#menuHelp
     (Menu new fromLiteralArrayEncoding:(ToolApplicationModel menuHelp)) startUp
    "

    <resource: #menu>

    ^
     
       #(#Menu
          
           #(
             #(#MenuItem
                #label: 'Documentation'
                #translateLabel: true
                #value: #openDocumentation
                #activeHelpKey: #helpTutorial
                #enabled: #valueOfHavingDocumentation
            )
             #(#MenuItem
                #label: '-'
            )
             #(#MenuItem
                #label: 'Show Help Texts'
                #translateLabel: true
                #activeHelpKey: #helpShowHelp
                #indication: #showingHelp:
            )
                       #(#MenuItem
                          #label: '-'
                      )
                       #(#MenuItem
                          #label: 'About this Application...'
                          #translateLabel: true
                          #value: #openAboutThisApplication
                          #activeHelpKey: #aboutThisAppliaction
                      )
          ) nil
          nil
      )

    "Modified: / 31.7.1998 / 18:19:42 / cg"
!

xxxmenuFont
    "This resource specification was automatically generated
     by the MenuEditor of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the MenuEditor may not be able to read the specification."

    "
     MenuEditor new openOnClass:ToolApplicationModel andSelector:#menuFont
     (Menu new fromLiteralArrayEncoding:(ToolApplicationModel menuFont)) startUp
    "

    <resource: #menu>

    ^
     
       #(#Menu
          
           #(
             #(#MenuItem
                #label: 'Label...'
                #translateLabel: true
                #value: #chooseFont:
                #argument: 'Label Font'
            )
             #(#MenuItem
                #label: 'Button...'
                #translateLabel: true
                #value: #chooseFont:
                #argument: 'Button Font'
            )
             #(#MenuItem
                #label: 'Edit...'
                #translateLabel: true
                #value: #chooseFont:
                #argument: 'Edit Font'
            )
             #(#MenuItem
                #label: 'List...'
                #translateLabel: true
                #value: #chooseFont:
                #argument: 'List Font'
            )
             #(#MenuItem
                #label: 'Tree List...'
                #translateLabel: true
                #value: #chooseFont:
                #argument: 'Tree List Font'
            )
             #(#MenuItem
                #label: 'Note Book...'
                #translateLabel: true
                #value: #chooseFont:
                #argument: 'Note Book Font'
            )
             #(#MenuItem
                #label: 'Framed Box...'
                #translateLabel: true
                #value: #chooseFont:
                #argument: 'Framed Box Font'
            )
             #(#MenuItem
                #label: '-'
            )
             #(#MenuItem
                #label: 'Reset'
                #translateLabel: true
                #value: #resetFonts
            )
          ) nil
          nil
      )

    "Modified: / 31.7.1998 / 18:55:34 / cg"
! !

!ToolApplicationModel class methodsFor:'queries'!

isVisualStartable
    "return true, if this application can be started via #open.
     (to allow start of a change browser via double-click in the browser)"

    self == ToolApplicationModel ifTrue:[^false].
    ^super isVisualStartable
! !

!ToolApplicationModel class methodsFor:'startup / release'!

preSnapshot
    "removes the clipboard before snap shoting"

    clipboard := nil
!

uninitialize
    "resets the class instance variables"

    settings := instances := showingHelp := history := clipboard := nil

    "
     self withAllSubclasses do:[:c | c uninitialize]
    "

    "Modified: / 4.2.1999 / 15:37:43 / cg"
! !

!ToolApplicationModel methodsFor:'aspects'!

acceptChannel
    "returns a valueHolder which can be set to true to force all inputFields
     to accept their value."

    |holder|

    (holder := builder bindingAt:#acceptChannel) isNil ifTrue:[
        builder aspectAt:#acceptChannel put:(holder :=  ValueHolder with:false).
    ].
    ^ holder

    "Modified: / 21.5.1998 / 03:39:49 / cg"
!

modifiedChannel
    "returns a valueHolder which is set to true, whenever any of
     my editfields is modified - this is hooked to
     the valueOfEnablingCommitButtons channel"

    |holder|

    (holder := builder bindingAt:#modifiedChannel) isNil ifTrue:[
        builder aspectAt:#modifiedChannel put:(holder :=  ValueHolder with:false).
        holder 
            onChangeEvaluate:[
                holder value ifTrue:[
                    self valueOfEnablingCommitButtons value:true.
                ]
               ]
    ].
    ^ holder

    "Modified: / 21.5.1998 / 03:39:49 / cg"
!

valueOfCanCopy
    "returns whether can copy as value holder"

    ^ builder booleanValueAspectFor:#valueOfCanCopy

    "Modified: / 21.5.1998 / 03:39:05 / cg"
!

valueOfCanCut
    "returns whether can paste as value holder"

    ^ builder booleanValueAspectFor:#valueOfCanCut

    "Modified: / 21.5.1998 / 03:39:13 / cg"
!

valueOfCanPaste
    "returns whether can paste as value holder"

    ^ builder booleanValueAspectFor:#valueOfCanPaste

    "Modified: / 21.5.1998 / 03:39:22 / cg"
!

valueOfCanUndo
    "returns whether can undo as value holder"

    ^ builder booleanValueAspectFor:#valueOfCanUndo

    "Modified: / 21.5.1998 / 03:39:31 / cg"
!

valueOfEnablingCommitButtons
    "returns the enabling of the commit of this tool as value holder"

    ^ builder booleanValueAspectFor:#valueOfEnablingCommitButtons

    "Modified: / 21.5.1998 / 03:39:49 / cg"
!

valueOfHavingDocumentation
    "returns the enabling of having documentation for this tool as value holder"

    ^ (builder booleanValueAspectFor:#valueOfHavingDocumentation) value: (self class includesSelector: #openDocumentation)
!

valueOfInfoLabel
    "returns the info label as value holder"

    ^ builder nilValueAspectFor:#valueOfInfoLabel

    "Modified: / 21.5.1998 / 03:40:34 / cg"
!

valueOfTimeLabel
    "returns the time label as value holder"

    ^ builder nilValueAspectFor:#valueOfTimeLabel

    "Modified: / 21.5.1998 / 03:40:42 / cg"
! !

!ToolApplicationModel methodsFor:'clipboard'!

clipboard
    "returns the clipboard"

    ^self class clipboard
!

clipboard: anEditObject
    "sets the clipboard and the value holder for can pasting"

    self class clipboard: anEditObject.

    self valueOfCanPaste value: anEditObject notNil
! !

!ToolApplicationModel methodsFor:'clock handling'!

startClock
    "/ start the time-block
    self startClockOnTimedBlock:[self updateTime]. 
!

startClockOnTimedBlock: aBlock
    "sets and starts the (optional) time block"

    timeBlock := aBlock.
    aBlock value.

    "Modified: / 31.7.1998 / 18:46:28 / cg"
!

stopClock
    "/ stop the time-block

    timeBlock notNil ifTrue:[
        Processor removeTimedBlock:timeBlock.
        timeBlock := nil.
    ].
    self valueOfTimeLabel value:''
!

updateTime
    "update my time-field"

    |now hours minutes suffix|

    now := Time now.

    hours := now hours.
    minutes := now minutes printString.
    suffix := ' '.

    (resources at:'TIME_FORMAT' ifAbsent:12) == 12 ifTrue:[
        suffix := ' am '.  
        hours > 12 ifTrue: [hours := hours - 12. suffix := ' pm '].
    ].

    minutes size = 1 ifTrue: [minutes := '0', minutes].

    self valueOfTimeLabel value:(hours printString, ':', minutes, suffix).

    timeBlock notNil ifTrue:[
        Processor removeTimedBlock:timeBlock.
        Processor addTimedBlock:timeBlock afterSeconds:5
    ]
! !

!ToolApplicationModel methodsFor:'defaults'!

aboutImage
    "the image to be displayed in my about-box;
     If nil is returned, thhe ST/X default image is used."

    ^ nil

    "Created: / 25.7.1998 / 20:33:50 / cg"
! !

!ToolApplicationModel methodsFor:'help'!

aboutThisApplicationText
    "text for an about box for this application.
     Extracts some common info (i.e. class revision, author etc.) from
     the classes documentation."

    |rev myClass clsRev msg|

    rev := ''.
    myClass := self class.

    (clsRev := myClass revision) notNil ifTrue: [rev := '  (rev: ', clsRev printString, ')'].

    msg := '\The application\\' withCRs
        , myClass name asBoldText, rev
        , '\\has been designed and implemented by: \' withCRs
        , ((self class authorLinesForAboutBox collect:[:l | '- ' , l]) asStringWith:$\) withCRs
        , '\\' withCRs.
    msg := msg withTabsExpanded.
    ^msg

    "Modified: / 12.3.1999 / 00:12:13 / cg"
    "Created: / 13.11.2001 / 12:34:19 / cg"
!

defaultInfoLabel
    "returns the default info label; here an empty string"

    ^ ''

    "Modified: / 31.7.1998 / 18:44:55 / cg"
!

openAbout
    "opens an about box (on ST/X)"

    self class openAboutSTX

    "Modified: / 14.8.1998 / 13:17:57 / cg"
!

openHTMLDocument: aHTMLFilename
    "opens a HTML browser on aHTMLFilename"

    HTMLDocumentView openFullOnDocumentationFile: aHTMLFilename
!

showHelp:aHelpText for:view
    "displays aHelpText in the info label"

"/    masterApplication notNil ifTrue:[
"/        ^ masterApplication showHelp:aHelpText for:view
"/    ].
"/
    aHelpText isNil 
        ifTrue:  [self updateInfoLabel] 
        ifFalse: [self valueOfInfoLabel value: (aHelpText asString printString readStream through: $.)].

    "Created: / 20.6.1998 / 16:37:23 / cg"
    "Modified: / 31.7.1998 / 18:40:19 / cg"
!

showingHelp
    "returns true, if automatic display of help is enabled for this
     application; false otherwise"

    |currentActiveHelp|

    currentActiveHelp := ActiveHelp currentHelpListener.

    ^ self class showingHelp 
      and: [currentActiveHelp notNil and: [
                currentActiveHelp interestedIn: self window]]

"/    ^ self class showingHelp 
"/      and: [currentActiveHelp notNil and: [masterApplication notNil 
"/                ifFalse: [currentActiveHelp interestedIn: self window]
"/                ifTrue:  [currentActiveHelp interestedIn: masterApplication window]]]

    "Created: / 31.7.1998 / 17:57:03 / cg"
    "Modified: / 22.8.1998 / 00:06:38 / cg"
!

showingHelp: aValue
    "toggles showing help for this application"

    (self class showingHelp: aValue)
        ifTrue:  [ActiveHelp startFor:self]
        ifFalse: [ActiveHelp stopFor:self. self updateInfoLabel]

    "Created: / 31.7.1998 / 18:20:44 / cg"
    "Modified: / 31.7.1998 / 18:44:39 / cg"
!

updateInfoLabel
    "updates the info label at the bottom of this application"

    self valueOfInfoLabel value: self defaultInfoLabel

    "Modified: / 31.7.1998 / 18:44:46 / cg"
!

xxopenAboutThisApplication
    "opens an about box for this application.
     Extracts some common info (i.e. class revision, author etc. from
     the classes documentation."

    |rev box myClass clsRev image msg|

    rev := ''.
    myClass := self class.

    (clsRev := myClass revision) notNil ifTrue: [rev := '  (rev: ', clsRev printString, ')'].

    msg := '\The application\\' withCRs
        , myClass name asBoldText, rev
        , '\\has been designed and implemented by: \' withCRs
        , ((self class authorLinesForAboutBox collect:[:l | '- ' , l]) asStringWith:$\) withCRs
        , '\\' withCRs.
    msg := msg withTabsExpanded.

    AboutBox isNil ifTrue:[
        "/ this handles bad installations of ST/X,
        "/ where the AboutBox is missing.
        "/ (May vanish in the future)
        ^ self information:msg
    ].

    box := AboutBox title:msg.

    image := self aboutImage.
    image notNil ifTrue:[
        box image:image
    ].
    box   label:(resources string:self aboutThisApplicationLabel).
    box   autoHideAfter:10 with:[].
    box   showAtPointer.

    "Modified: / 13.11.2001 / 12:31:28 / cg"
    "Created: / 13.11.2001 / 12:34:27 / cg"
! !

!ToolApplicationModel methodsFor:'history'!

addToHistory: aHistoryEntry
    "adds aHistoryEntry (format: loadMessage -> evalString) at the top of the history,
     and checks for maximum size of the history"

    aHistoryEntry key size = 0 ifTrue: [^nil].
    self history remove: (self history detect: [:histEntry| histEntry key = aHistoryEntry key] ifNone: nil) ifAbsent: nil.
    self history addFirst: aHistoryEntry.
    [self history size > self class historyMaxSize] whileTrue: [self history removeLast]
!

emptyHistory
    "removes all history entries"

    ^self history removeAll
!

history
    "returns the history from tool class"

    ^self class getHistory
!

historyEntries
    "returns the history entries, i.e. the evaluatable values containing the information"

    ^self history collect: [:asso| asso key]
!

menuHistory
    "returns a history submenu"

    |menu|

    menu := Menu new receiver: self.
    (self history collect: [:histEntry| histEntry value]) asSet asOrderedCollection do:
    [:historyEntryType|    
        menu addItemGroup:
                ((self history 
                    select: [:histEntry| 
                        histEntry value = historyEntryType]) 
                    collect: [:histEntry|  
                        MenuItem new 
                            label: histEntry key printString; 
                            value: histEntry value; 
                            argument: histEntry key; 
                            activeHelpKey: #historyMenuItem]).
    ]. 

    menu addItem: (MenuItem new 
                        label: 'Empty History'; 
                        value: #emptyHistory; 
                        activeHelpKey: #historyEmptyMenu;
                        translateLabel:true).

    menu findGuiResourcesIn:self.
    ^menu

    "Modified: / 29.7.1998 / 11:40:11 / cg"
!

removeFromHistory: aHistoryEntry
    "removes aHistoryEntry from the history"

    self history remove: (self history detect: [:histEntry| histEntry key = aHistoryEntry key] ifNone: nil) ifAbsent: nil.
    [self history size > self class historyMaxSize] whileTrue: [self history removeLast]
! !

!ToolApplicationModel methodsFor:'initialization'!

initialize
    super initialize.
    self createBuilder

    "Created: / 20.6.1998 / 14:51:29 / cg"
! !

!ToolApplicationModel methodsFor:'misc'!

clearModifiedFlag
    "clear the modifiedChannel.
     That one is hooked to components in the right noteBook,
     and will be set, when any value is modified.
     (to enable OK and cancel buttons)"

    self modifiedChannel value:false
! !

!ToolApplicationModel methodsFor:'queries'!

allToolInstances
    "returns all instances of this tool class"

    ^ self class allToolInstances

    "Modified: / 4.2.1999 / 15:34:58 / cg"
!

preferredExtent
    "preferred extent of my window; top/main menu and top toolbar (named by 'menuToolbarView')
     will be considered"

    |window menu menuToolbarView preferredExtentOfWindow|

    window          := self builder window. 
    menu            := window subViews first.
    menuToolbarView := builder componentAt: #menuToolbarView.

    preferredExtentOfWindow := window extent.
    menu class = MenuPanel ifTrue: [preferredExtentOfWindow := preferredExtentOfWindow max: menu preferredExtent].
    menuToolbarView notNil ifTrue: [preferredExtentOfWindow := preferredExtentOfWindow max: menuToolbarView preferredExtent].

    ^window minExtent max: (preferredExtentOfWindow min: window maxExtent)
! !

!ToolApplicationModel methodsFor:'startup / release'!

closeDownViews
    "uninitializes and closes this tool"

    self uninitialize.
    super closeDownViews
!

postOpenWith:aBuilder
    "starts the active help for this tool (if turned on in the settings)
     and updates the info label and the fonts (from the settings)"

    super postOpenWith:aBuilder.

    self updateInfoLabel.
"/    self updateFonts.
    self showingHelp:true

    "Modified: / 31.7.1998 / 18:17:34 / cg"
!

restarted
    "sent by my windowGroup, when restarted from an image.
     restart timed block if defined.
    "
    timeBlock notNil ifTrue:[
        Processor removeTimedBlock:timeBlock.
        self startClockOnTimedBlock:timeBlock.
    ].
    super restarted.
!

uninitialize
    "stops active help for this tool and (if defined) removes the time block"

    self showingHelp ifTrue: [ActiveHelp stopFor: self].
    timeBlock notNil ifTrue: [Processor removeTimedBlock:timeBlock. timeBlock := nil].

    "Modified: / 31.7.1998 / 18:46:21 / cg"
! !

!ToolApplicationModel class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libview2/ToolApplicationModel.st,v 1.108 2002-02-25 20:01:24 cg Exp $'
! !
